/*
 *  Copyright (C) 2012-2017 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 *  \file   mcspiSample_io.c
 *
 *  \brief  McSPI evm to evm communication sample application
 *
 *          This file demonstrates the use of Mcspi by using an
 *          EVM to EVM communication setup.This file configures one of the evm
 *          in master mode.
 */

/* ========================================================================== */
/*                          INCLUDE FILES                                     */
/* ========================================================================== */

#include <stdio.h>
#include <string.h>
#include <xdc/std.h>
#include <ti/sysbios/io/GIO.h>
#include <ti/sysbios/BIOS.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/knl/Task.h>
#include <xdc/runtime/IHeap.h>
#include <ti/sysbios/heaps/HeapMem.h>
#include <ti/drv/bsp_lld/mcspi/bsp_mcspi.h>
#include <xdc/runtime/Memory.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <xdc/std.h>
#include <stdio.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/io/GIO.h>
#include <ti/sysbios/io/IOM.h>
#include <xdc/runtime/Log.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/heaps/HeapMem.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/drv/vps/include/common/bsp_types.h>
#include <ti/drv/vps/include/common/trace.h>
#include <ti/drv/vps/include/common/bsp_config.h>
#include <ti/drv/vps/include/common/bsp_utils.h>
#include <ti/drv/vps/include/common/bsp_common.h>
#include <ti/drv/bsp_lld/mcspi/bsp_mcspi.h>
#include <ti/drv/vps/include/platforms/bsp_platform.h>
#include <ti/drv/vps/include/boards/bsp_board.h>
#include <ti/drv/vps/include/devices/bsp_device.h>
#include <ti/drv/bsp_lld/i2c/bsp_i2c.h>
#include <ti/drv/vps/include/fvid2/fvid2.h>
#include <ti/drv/vps/examples/utility/bsputils_mem.h>
#include <ti/drv/vps/examples/utility/bsputils_prf.h>
#include <ti/drv/vps/examples/utility/bsputils_app.h>
#include <src/rtos/utils_common/include/utils_dma.h>
#include <ti/csl/csl_mcspi.h>
#include <ti/csl/soc.h>
#include "mcspiSample_io.h"

/* ========================================================================== */
/*                          LOCAL FUNCTION PROTOTYPES                         */
/* ========================================================================== */
EDMA3_DRV_Handle edma3init(UInt32 edma3Id, EDMA3_DRV_Result *);
Int32 App_mcspiDefaultInit(UInt32 isI2cInitReq);
static Int32 App_mcspiDefaultDeInit(UInt32 isI2cInitReq);
void SPI_init(void);
/* ========================================================================== */
/*                            MACRO DEFINITONS                                */
/* ========================================================================== */

/* ========================================================================== */
/*                              GLOBAL VARIABLES                              */
/* ========================================================================== */

EDMA3_DRV_Handle      gEdmaHandle;
UInt32       gIsI2cInitReq;
extern const IOM_Fxns Mcspi_IOMFXNS;
HeapMem_Handle myHeap;
Bool garbej_on_boot = TRUE;
//static I2c_DevInitParams gI2cDevInitParams[BSP_DEVICE_I2C_INST_ID_MAX];

/*
 * Buffers placed in external memory are aligned on a 128 bytes boundary.
 * In addition, the buffer should be of a size multiple of 128 bytes for
 * the cache to work optimally.
 */

//#define BUFLEN          (1024 * 4) /* Buffer size */
#define BUFLEN          (16 * 1) /* Buffer size */
#define BUFALIGN        128        /* Alignment of buffer for use of L2 cache */
#define ITERATION_CNT   100        /* Number of Iterations */
//#define NUM_BUFS        3          /* Num Bufs to be issued and reclaimed */
#define NUM_BUFS        1         /* Num Bufs to be issued and reclaimed *///value is must be two
/* handle to the input and output streams   */
GIO_Handle mcspiHandle = NULL;

/* Global SPI init config data structure    */
Mcspi_Params   mcspiPrms;

Mcspi_DataParam       issueDataparam[NUM_BUFS];

//Mcspi_DataParam       issueDataparam1[NUM_BUFS];

/* Buffer alignement is required when working in DMA Mode */
#pragma DATA_ALIGN(rxbuf, BUFALIGN);
Ptr  rxbuf[NUM_BUFS];

/* Buffer alignement is required when working in DMA Mode */
#pragma DATA_ALIGN(txbuf, BUFALIGN);
Ptr  txbuf[NUM_BUFS];

#pragma DATA_ALIGN(rxbuf, BUFALIGN);
Ptr  rxbuf1[NUM_BUFS];

/* Buffer alignement is required when working in DMA Mode */
#pragma DATA_ALIGN(txbuf, BUFALIGN);
Ptr  txbuf1[NUM_BUFS];

Bool failFlag = FALSE;

/* Function prototype */
//#if 0
static void createStream(void);
static void prime(void);
static void memory_allocated(void);
//static void prime1(void);
//#endif
/* ========================================================================== */
/*                           FUNCTION DEFINITIONS                             */
/* ========================================================================== */

/*
 * ======== createStream ========
 */
//#if 0
static void createStream(void)
{
    GIO_Params       ioParams;
    Mcspi_ChanParams chanParams;
    Error_Block      eb;

    Error_init(&eb);

    /*
     * Initialize channel attributes.
     */
    GIO_Params_init(&ioParams);

    /* update the edma Handle                                                 */
    chanParams.hEdma = gEdmaHandle;
    chanParams.chipSelTimeControl = MCSPI_CLK_CYCLE0;
    chanParams.fifoEnable         = (UInt32) TRUE;
    chanParams.spiChipSelectHold  = (UInt32) TRUE;
    chanParams.chanNum = 0;

    /* If cross bar events are being used then make isCrossBarIntEn = TRUE and
     * choose appropriate interrupt number to be mapped (assign it to
     * intNumToBeMapped)
     */
    /* Cross bar evt disabled */
    chanParams.crossBarEvtParam.isCrossBarIntEn  = (UInt32) FALSE;
    chanParams.crossBarEvtParam.intNumToBeMapped = 0xFF;  /* Invalid number */

    ioParams.chanParams = (Ptr) & chanParams;
    ioParams.model      = GIO_Model_ISSUERECLAIM;
    ioParams.numPackets = NUM_BUFS + 1;
//    ioParams.sync=NULL;
    mcspiHandle = GIO_create("/mcspi0", GIO_INOUT, &ioParams, &eb);
    if (mcspiHandle == NULL)
    {
        System_printf("\r\nCreate input stream FAILED.\r\n");
        BIOS_exit(0);
    }
    System_printf("\r\nCreate input stream completed.\r\n");
}
//#endif
/*
 * \brief   Function to submit request the driver.Depending on the macro
 *          "NUM_BUFS" either the driver can be buffered with multiple requests
 *          or only one buffer by using the NUM_BUFS as 1.
 *
 * \param   None
 *
 * \return  None
 */


static void memory_allocated(void)
{
    Error_Block  eb;
    Int32        count = 0;
    IHeap_Handle iheap;

    iheap = HeapMem_Handle_to_xdc_runtime_IHeap(myHeap);
    Error_init(&eb);

    /* Allocate buffers for the GIO buffer exchanges                          */
    for (count = 0; count < (NUM_BUFS); count++)
    {
        rxbuf[count] = (Ptr) Memory_calloc(iheap, BUFLEN, BUFALIGN, &eb);
        if (NULL == rxbuf[count])
        {
            System_printf("\r\nMEM_calloc failed.\r\n");
        }
        txbuf[count] = (Ptr) Memory_calloc(iheap, BUFLEN, BUFALIGN, &eb);
        if (NULL == txbuf[count])
        {
            System_printf("\r\nMEM_calloc failed.\r\n");
        }
    }
}
//if 0
static void prime(void)
{

    Int32        count = 0;
    UInt32       tempCount = 0;
    UInt32       size      = 0;
    Int          status    = IOM_COMPLETED;
    UInt8           j;
    UInt8 spi_dummy_data[13]={0x00,0x20,0x09,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F};
    unsigned char data = 0x0F;

        if(garbej_on_boot == TRUE)
        {
           for (count = 0; count < (NUM_BUFS); count++)
           {
                for (tempCount = 0; tempCount < BUFLEN; tempCount++)
                {
                    ((Uint8 *) txbuf[count])[tempCount] = data;
                }

           }
           garbej_on_boot =FALSE;
        }
        else
        {
            for (count = 0; count < (NUM_BUFS); count++)
           {
            for (tempCount = 0,j=0; tempCount < BUFLEN; tempCount++)
            {
               ((Uint8 *) txbuf[count])[tempCount] = spi_dummy_data[j++];
               if(j==13)
                   j=0;
            }
           }
        }
    for (count = 0; count < NUM_BUFS; count++)
    {
        issueDataparam[count].bufLen    = BUFLEN;
        issueDataparam[count].inBuffer  = rxbuf[count];
        issueDataparam[count].outBuffer = txbuf[count];

        size = issueDataparam[count].bufLen;
        System_printf("\r\n before GIO_issue \r\n");
        /* Issue the first & second empty buffers to the input stream         */
        status = GIO_issue(mcspiHandle, &issueDataparam[count], size, NULL);

        if (status != IOM_PENDING && status != IOM_COMPLETED)
        {
            System_printf("\r\nsatus =%d\r\n",status);
            System_printf("\r\nFailed to issue empty buffer to stream\r\n");
        }
        System_printf("SPI TDA2x-> MCU write frame replay\r\n");
        for(j=0;j<BUFLEN;j++ )
        {
            System_printf("SPI Data[%d byte]=0x%x\r\n", j,issueDataparam[count].outBuffer[j]);

        }
        System_printf("SPI End frame\r\n");
    }
    System_printf("\r\n prime completed\r\n");

}
#if 0
static void prime1(void)
{

    Error_Block  eb;
    Int32        count = 0;
    IHeap_Handle iheap;
    UInt32       tempCount = 0;
    UInt32       size      = 0;
    Int          status    = IOM_COMPLETED;

    iheap = HeapMem_Handle_to_xdc_runtime_IHeap(myHeap);
    Error_init(&eb);

    /* Allocate buffers for the GIO buffer exchanges                          */
    for (count = 0; count < (NUM_BUFS); count++)
    {
        rxbuf[count] = (Ptr) Memory_calloc(iheap, BUFLEN, BUFALIGN, &eb);
        if (NULL == rxbuf[count])
        {
            System_printf("\r\nMEM_calloc failed.\r\n");
        }

        txbuf[count] = (Ptr) Memory_calloc(iheap, BUFLEN, BUFALIGN, &eb);
        if (NULL == txbuf[count])
        {
            System_printf("\r\nMEM_calloc failed.\r\n");
        }
    }

    /* Fill the buffers with known data and transmit the same and check if the*
     * same pattern is received on the other EVM                              */
    for (count = 0; count < (NUM_BUFS); count++)
    {
        for (tempCount = 0; tempCount < BUFLEN; tempCount++)
        {
//            ((Uint8 *) txbuf[count])[tempCount] = tempCount;
            ((Uint8 *) txbuf[count])[tempCount] =238;
        }
    }
    for (count = 0; count < NUM_BUFS; count++)
    {
        issueDataparam1[count].bufLen    = BUFLEN;
        issueDataparam1[count].inBuffer  = rxbuf[count];
        issueDataparam1[count].outBuffer = txbuf[count];

        size = issueDataparam1[count].bufLen;

        /* Issue the first & second empty buffers to the input stream         */
        status = GIO_issue(mcspiHandle, &issueDataparam1[count], size, NULL);

        if (status != IOM_PENDING && status != IOM_COMPLETED)
        {
            System_printf("\r\nsatus =%d\r\n",status);
            System_printf("\r\nFailed to issue empty buffer to stream\r\n");
        }
    }
    System_printf("\r\n prime completed\r\n");
    GIO_reclaim(mcspiHandle, (Ptr *)&issueDataparam1[count], NULL, NULL);
//    status = GIO_reclaim(mcspiHandle, (Ptr *)&issueDataparam[i], NULL, NULL);
}
#endif
void spiSampleTask(void)
{
      Int32 retVal = FVID2_SOK;
      Bsp_PlatformInitParams platInitPrms;

    /* isPinMuxSettingReq parameter will be initialised to TRUE */
    BspPlatformInitParams_init(&platInitPrms);
//
    platInitPrms.isAllMcSPIInitReq  = TRUE;
    platInitPrms.isPinMuxSettingReq = TRUE;

    /* Initialize pinmux and evm related configurations */
    Bsp_platformInit(&platInitPrms);
    /* System init */
    gIsI2cInitReq = FALSE;
    retVal       = App_mcspiDefaultInit(gIsI2cInitReq);
    if (retVal != FVID2_SOK)
    {
        System_printf("Error: : System Init Failed!!!\r\n");
    }
    /* Set the pin Mux */
    Bsp_boardSetPinMux(BSP_DRV_ID_MCSPI, BSP_DEVICE_MCSPI_INST_ID_0,
                       BSP_BOARD_MODE_DEFAULT);
//    Bsp_boardSetPinMux(BSP_DRV_ID_MCSPI, BSP_DEVICE_MCSPI_INST_ID_1,
//                       BSP_BOARD_MODE_DEFAULT);

    /* Set the board muxes */
    Bsp_boardSelectDevice(BSP_DRV_ID_MCSPI, BSP_DEVICE_MCSPI_INST_ID_0);
//    Bsp_boardSelectDevice(BSP_DRV_ID_MCSPI, BSP_DEVICE_MCSPI_INST_ID_1);
    /* call the function for the SPI application test                     */

//    start_spi_sample();
//
//    App_mcspiDefaultDeInit(gIsI2cInitReq);
//
//    /* Start the Heart Beat Print                                         */
//    tskHeartBeat();
//
//    return;
}

/*
 * \brief   This function demostrates the use of Mcspi using an EVM to EVM
 *          communication setup.
 *
 * \param   None
 *
 * \return  None
 */
void start_spi_sample_new(Void)
{
    System_printf("\r\nIn start SPI sample new\r\n");
    volatile Int32   i32Count  = 0;

//    Mcspi_DataParam  dataparam = {0};
//    Mcspi_DataParam *dataparam = {0};
    UInt32           tempCount = 0;
    UInt32           i /*loopcount = 0*/;
    UInt32             k,j;
//    UInt32           dataMismatchError = 0;
//    UInt8            temp8InBufVal = 0, temp8OutBufVal = 0;
//    UInt16           temp16InBufVal = 0, temp16OutBufVal = 0;
//    UInt32           temp32InBufVal = 0, temp32OutBufVal = 0;
//    UInt32           maskVal        = 0;
    UInt8               SPIdata[13] = {0};
//    UInt8            SPIReplaydata[13] = {0};
    Int status = IOM_COMPLETED;
//    size_t       size      = 0;
//    Sleep(1000);
    SPI_init();
    spiSampleTask();
//    createStream();


//    EDMA3_DRV_Result edmaResult = 0;
    //Utils_dmaInit();
//    gEdmaHandle = Utils_dmaGetEdma3Hndl(0);
////    gEdmaHandle = edma3init(0, &edmaResult);
//
//    if (gEdmaHandle != NULL)
//    {
//        /* Report EDMA Error */
//        System_printf("\r\nEDMA driver initialization FAIL\r\n");
//    }
//    else
//    {
//        System_printf("\r\nEDMA driver initialization PASS.\r\n");
//    }

    /* Call createStream function to create I/O streams                       */
    createStream();
    memory_allocated();
for(k=0;k<1;k++)
{
    prime();

    for (i = 0; i < NUM_BUFS; i++)
    {

        status = GIO_reclaim(mcspiHandle, (Ptr *)&issueDataparam[i], NULL, NULL);
        if (IOM_COMPLETED != status)
        {
            System_printf("Iteration %d\r\n", i32Count);

            System_printf(
                "Error reclaiming empty buffer from the streams %x"
                " error = 0x%d\r\n",
                ((Uint8) (issueDataparam[i].outBuffer[i32Count])),
                status);
            break;
        }

        for (tempCount = 0; tempCount < BUFLEN; )
        {
//            System_printf("Out Buffer Data %x\r\n", ((Uint8) (issueDataparam[i].outBuffer[tempCount])));
//            System_printf("In Buffer Data [byte %d]=0x%x\r\n", tempCount,((Uint8) (issueDataparam[i].inBuffer[tempCount])));
            if (mcspiPrms.spiHWCfgData.configChfmt[0].charLength <= 8)
            {
                if( (issueDataparam[i].inBuffer[tempCount] == 0x00) && (issueDataparam[i].inBuffer[tempCount+1] == 0x22))
                {

                    SPIdata[0]=issueDataparam[i].inBuffer[tempCount];
                    SPIdata[1]=issueDataparam[i].inBuffer[tempCount + 1];
                    SPIdata[2]=issueDataparam[i].inBuffer[tempCount + 2];
                    SPIdata[3]=issueDataparam[i].inBuffer[tempCount + 3];
                    SPIdata[4]=issueDataparam[i].inBuffer[tempCount + 4];
                    SPIdata[5]=issueDataparam[i].inBuffer[tempCount + 5];
                    SPIdata[6]=issueDataparam[i].inBuffer[tempCount + 6];
                    SPIdata[7]=issueDataparam[i].inBuffer[tempCount + 7];
                    SPIdata[8]=issueDataparam[i].inBuffer[tempCount + 8];
                    SPIdata[9]=issueDataparam[i].inBuffer[tempCount + 9];
                    SPIdata[10]=issueDataparam[i].inBuffer[tempCount + 10];
                    SPIdata[11]=issueDataparam[i].inBuffer[tempCount + 11];
                    SPIdata[12]=issueDataparam[i].inBuffer[tempCount + 12];

                   System_printf("SPI Master-> TDA2x write frame\r\n");
                   for(j=0;j<13;j++ )
                   {
                      System_printf("SPI Data[%d byte]=%x\r\n", j,SPIdata[j]);

                   }
                   System_printf("SPI End frame\r\n");

                }
                else if((issueDataparam[i].inBuffer[tempCount] == 0x01) && (issueDataparam[i].inBuffer[tempCount+1] == 0x22))
                {
                    SPIdata[0]=issueDataparam[i].inBuffer[tempCount];
                    SPIdata[1]=issueDataparam[i].inBuffer[tempCount + 1];
                    SPIdata[2]=issueDataparam[i].inBuffer[tempCount + 2];
                    SPIdata[3]=issueDataparam[i].inBuffer[tempCount + 3];
                    SPIdata[4]=issueDataparam[i].inBuffer[tempCount + 4];
                    SPIdata[5]=issueDataparam[i].inBuffer[tempCount + 5];
                    SPIdata[6]=issueDataparam[i].inBuffer[tempCount + 6];
                    SPIdata[7]=issueDataparam[i].inBuffer[tempCount + 7];
                    SPIdata[8]=issueDataparam[i].inBuffer[tempCount + 8];
                    SPIdata[9]=issueDataparam[i].inBuffer[tempCount + 9];
                    SPIdata[10]=issueDataparam[i].inBuffer[tempCount + 10];
                    SPIdata[11]=issueDataparam[i].inBuffer[tempCount + 11];
                    SPIdata[12]=issueDataparam[i].inBuffer[tempCount + 12];

                    System_printf("SPI start read frame\r\n");
                    for(j=0;j<13;j++ )
                    {
                       System_printf("SPI Data[%d byte]=%x\r\n", j,SPIdata[j]);

                    }
                    System_printf("SPI End frame\r\n");
                }
            }
            else
            {
                System_printf("Invalid Word Length\r\n");

            }
            System_printf("in temp count=%d\r\n",tempCount);
            tempCount++;
        }
        System_printf("in time NUM_BUFS = %d\r\n",i);
    }
//    System_printf("loop time = %d\r\n",k);
}
    gIsI2cInitReq = FALSE;
    GIO_delete(&mcspiHandle);
    App_mcspiDefaultDeInit(gIsI2cInitReq);
    return;
}

void user_mcspi_init(void)
{
  //  Int32 opMode;
    Mcspi_init();

//    mcspiPrms = Mcspi_PARAMS;
//    System_printf("Please Connect the McSPI%d MISO (DAT0) to MOSI (DAT1)\r\n",
//                  (inst_num + 1U));
//    System_printf(
//        "And then enter the Operating mode\r\n0 - Polled Mode\
//            \r\n1 - Interrupt Mode\
//            \r\n2 - DMA Interrupt mode\r\n$>");
////    scanf("%d", &opMode);
//    if (opMode == 0)
//    {
//        mcspiPrms.opMode = MCSPI_OPMODE_POLLED;
//    }
//    else if (opMode == 1)
//    {
        mcspiPrms.opMode = MCSPI_OPMODE_INTERRUPT;
//    }
//    else if (opMode == 2)
//    {
//        mcspiPrms.opMode = MCSPI_OPMODE_DMAINTERRUPT;
//    }
//    else
//    {
//        mcspiPrms.opMode = MCSPI_OPMODE_POLLED;
//    }

    mcspiPrms.instNum         = 0;
    mcspiPrms.hwiNumber       = 7;
    mcspiPrms.enableCache     = (UInt32) TRUE;
    mcspiPrms.edma3EventQueue = 0;
    mcspiPrms.enableErrIntr   = (UInt32) FALSE;

    mcspiPrms.spiHWCfgData.masterOrSlave         = MCSPI_COMMMODE_SLAVE;
    mcspiPrms.spiHWCfgData.singleOrMultiChEnable = MCSPI_SINGLE_CHANNEL;
    mcspiPrms.spiHWCfgData.pinOpModes    = MCSPI_PINOPMODE_4PIN;
//    mcspiPrms.spiHWCfgData.fifoRxTrigLvl = 32;
//    mcspiPrms.spiHWCfgData.fifoTxTrigLvl = 32;
    mcspiPrms.spiHWCfgData.fifoRxTrigLvl = 16;
    mcspiPrms.spiHWCfgData.fifoTxTrigLvl = 16;
    mcspiPrms.spiHWCfgData.configChfmt[0].charLength = MCSPI_LEN_8BIT;
    mcspiPrms.spiHWCfgData.configChfmt[0].multiWordAccessEnable =
        (UInt32) FALSE;

    mcspiPrms.spiHWCfgData.configChfmt[0].spiChipSelectEnablePol =
            (UInt32) FALSE;

//    mcspiPrms.spiHWCfgData.configChfmt[0].spiChipSelectEnablePol =
//              (UInt32) TRUE;


    mcspiPrms.spiHWCfgData.configChfmt[0].clockMode           = MCSPI_MODE3;
//    mcspiPrms.spiHWCfgData.configChfmt[0].clockMode           = MCSPI_MODE1;
    mcspiPrms.spiHWCfgData.configChfmt[0].clockRatioExtension = 0;
    mcspiPrms.spiHWCfgData.configChfmt[0].spiWordInitDelay    = MCSPI_NO_DELAY;
    mcspiPrms.spiHWCfgData.configChfmt[0].trasmitReceiveMode  = MCSPI_BOTH_RXTX;
    mcspiPrms.spiHWCfgData.configChfmt[0].granularityEnable   = (UInt32) TRUE;
//    mcspiPrms.spiHWCfgData.configChfmt[0].busFreq             = 1000000;
    mcspiPrms.spiHWCfgData.configChfmt[0].busFreq             =  200000;
    mcspiPrms.spiHWCfgData.configChfmt[0].spienHighPolarity   = (UInt32) FALSE;
    mcspiPrms.spiHWCfgData.configChfmt[0].slaveModeChipSelect = MCSPI_SPIEN_0;
//    mcspiPrms.spiHWCfgData.configChfmt[0].spiDat0Dir          = MCSPI_IN;
//    mcspiPrms.spiHWCfgData.configChfmt[0].spiDat1Dir          = MCSPI_OUT;

    mcspiPrms.spiHWCfgData.configChfmt[0].spiDat0Dir          = MCSPI_OUT;
    mcspiPrms.spiHWCfgData.configChfmt[0].spiDat1Dir          = MCSPI_IN;

    if (MCSPI_OPMODE_POLLED == mcspiPrms.opMode)
    {
        System_printf("\r\nMcspi is configured in polled mode\r\n");
    }
    else if (MCSPI_OPMODE_INTERRUPT == mcspiPrms.opMode)
    {
        System_printf("\r\nMcspi is configured in interrupt mode\r\n");
    }
    else if (MCSPI_OPMODE_DMAINTERRUPT == mcspiPrms.opMode)
    {
        System_printf("\r\nMcspi is configured in dma mode\r\n");
    }
    else
    {
        System_printf("\r\nError: unknown mode of operation!!!!!!!!!!\r\n");
    }

}
void SPI_init(void)
{
System_printf("Running McSPI Loopback test.\r\n");
//    System_printf(
//        "Enter McSPI instance on which test to be run\
//        \r\n0 - McSPI1 instance\r\n1 - McSPI2 instance\r\n$>");
//    scanf("%d", &inst_num);
//    if (inst_num != 0 && inst_num != 1)
//    {
//        inst_num = 0;
//    }
    GIO_addDevice("/mcspi0", (xdc_Ptr) & Mcspi_IOMFXNS, &user_mcspi_init,
                  0, (xdc_Ptr) & mcspiPrms);
//    McSPIStartBitEnable(SOC_MCSPI1_BASE, MCSPI_CHANNEL_0);
//    McSPIStartBitPolarityConfig(SOC_MCSPI1_BASE,
//                                       MCSPI_START_BIT_POL_LOW,
//                                            MCSPI_CHANNEL_0);
}

Int32 App_mcspiDefaultInit(UInt32 isI2cInitReq)
{
    Int32  retVal = BSP_SOK;
    //UInt32 instCnt;
    Bsp_CommonInitParams    commonInitPrms;
    Bsp_BoardInitParams     boardInitPrms;
    Bsp_PlatformInitParams  platInitPrms;
    //Bsp_DeviceInitParams    deviceInitPrms;
    //lld_hsi2c_initParam_t   i2cInitParams[BSP_DEVICE_I2C_INST_ID_MAX];
    //const Bsp_BoardI2cData *i2cData;
    //Bsp_BoardI2cInstData   *i2cInstData;
    BspCommonInitParams_init(&commonInitPrms);
    retVal += Bsp_commonInit(&commonInitPrms);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: BSP Common Init failed!!\r\n");
    }

    BspBoardInitParams_init(&boardInitPrms);
    /* Override board detection if I2C is disabled */
    if (((Bool) TRUE) != ((Bool) isI2cInitReq))
    {
        boardInitPrms.boardId      = BSP_BOARD_UNKNOWN;
        boardInitPrms.baseBoardRev = BSP_BOARD_REV_UNKNOWN;
        boardInitPrms.dcBoardRev   = BSP_BOARD_REV_UNKNOWN;
    }
    retVal += Bsp_boardInit(&boardInitPrms);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: Board Init failed!!\r\n");
    }

    BspPlatformInitParams_init(&platInitPrms);
    retVal += Bsp_platformInit(&platInitPrms);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: Platform Init failed!!\r\n");
    }

    retVal += Fvid2_init(NULL);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: FVID2 Init failed!!\r\n");
    }

    /* Override I2C init for non-EVM builds */
    if (BSP_PLATFORM_ID_EVM != Bsp_platformGetId())
    {
        isI2cInitReq = FALSE;
    }

   /* if (((Bool) TRUE) == ((Bool) isI2cInitReq))
    {
        i2cData = Bsp_boardGetI2cData();
        GT_assert(BspAppTrace, (NULL != i2cData));
        GT_assert(BspAppTrace,
                  (i2cData->numInst <= BSP_DEVICE_I2C_INST_ID_MAX));
        GT_assert(BspAppTrace, (NULL != i2cData->instData));

        for (instCnt = 0U; instCnt < i2cData->numInst; instCnt++)
        {
            i2cInstData = &i2cData->instData[instCnt];
            GT_assert(BspAppTrace,
                      (i2cInstData->instId < BSP_DEVICE_I2C_INST_ID_MAX));
            i2cInitParams[instCnt].opMode       = HSI2C_OPMODE_INTERRUPT;
            i2cInitParams[instCnt].isMasterMode = TRUE;
            i2cInitParams[instCnt].is10BitAddr  = FALSE;
            i2cInitParams[instCnt].i2cBusFreq   =
                (lld_i2c_busspeed) i2cInstData->busClkKHz;
            i2cInitParams[instCnt].i2cIntNum      = i2cInstData->intNum;
            i2cInitParams[instCnt].i2cOwnAddr     = 0xCC;
            gI2cDevInitParams[instCnt].initParams = &i2cInitParams[instCnt];
            gI2cDevInitParams[instCnt].hsi2c_sem  =
                BspOsal_semCreate((Int32) 1, (Bool) TRUE);
            gI2cDevInitParams[instCnt].instId = i2cInstData->instId;
        }

        if (i2cData->numInst > 0)
        {
            retVal = I2c_GlobalInit(i2cData->numInst, &gI2cDevInitParams[0U]);
            if (BSP_SOK != retVal)
            {
                GT_0trace(BspAppTrace, GT_ERR, "Error: I2C Init failed!!\r\n");
            }
        }

        BspDeviceInitParams_init(&deviceInitPrms);
        deviceInitPrms.isI2cProbingReq = FALSE;
        retVal += Bsp_deviceInit(&deviceInitPrms);
        if (BSP_SOK != retVal)
        {
            GT_0trace(BspAppTrace, GT_ERR, "Error: Device Init failed!!\r\n");
        }
    }*/

//    retVal += BspUtils_memInit();
//    if (BSP_SOK != retVal)
//    {
//        GT_0trace(BspAppTrace, GT_ERR, "Error: App MEM Utils Init failed!!\r\n");
//    }
//
//    retVal += BspUtils_prfInit();
//    if (BSP_SOK != retVal)
//    {
//        GT_0trace(BspAppTrace, GT_ERR, "Error: App PRF Utils Init failed!!\r\n");
//    }

    retVal += BspUtils_appInit();
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: App Utils Init failed!!\r\n");
    }

    /* Print BSP version string and platform info*/
    GT_0trace(BspAppTrace, GT_INFO, " \r\n");
    GT_2trace(BspAppTrace, GT_INFO,
              " Build time            : [%s %s]\r\n", __TIME__, __DATE__);
    GT_1trace(BspAppTrace, GT_INFO,
              " BSP Version           : [%s]\r\n", Bsp_getVersionString());
    Bsp_platformPrintInfo();
    Bsp_boardPrintInfo();
    GT_0trace(BspAppTrace, GT_INFO, " \r\n");

    return (retVal);
}
/**
 *  App_mcspiDefaultDeInit
 *  \brief Application utils function to call all the default de-init
 *  functions.
 */
static Int32 App_mcspiDefaultDeInit(UInt32 isI2cDeInitReq)
{
    Int32  retVal = BSP_SOK;
//    const Bsp_BoardI2cData *i2cData;
//    UInt32 instCnt;

    retVal += BspUtils_appDeInit();
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: App Utils De-Init failed!!\r\n");
    }

    retVal += BspUtils_prfDeInit();
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR,
                  "Error: App PRF Utils De-Init failed!!\r\n");
    }

    retVal += BspUtils_memDeInit();
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR,
                  "Error: App MEM Utils De-Init failed!!\r\n");
    }

    /* Override I2C de-init for non-EVM builds */
    if (BSP_PLATFORM_ID_EVM != Bsp_platformGetId())
    {
        isI2cDeInitReq = FALSE;
    }

    if (((Bool) TRUE) == ((Bool) isI2cDeInitReq))
    {
//        i2cData = Bsp_boardGetI2cData();
//        GT_assert(BspAppTrace, (NULL != i2cData));
//        GT_assert(BspAppTrace,
//                  (i2cData->numInst <= BSP_DEVICE_I2C_INST_ID_MAX));
//
//        retVal += Bsp_deviceDeInit(NULL);
//        if (BSP_SOK != retVal)
//        {
//            GT_0trace(BspAppTrace, GT_ERR, "Error: Device De-Init failed!!\r\n");
//        }
//        if (i2cData->numInst > 0)
//        {
//            for (instCnt = 0U; instCnt < i2cData->numInst; instCnt++)
//            {
//                BspOsal_semDelete(&(gI2cDevInitParams[instCnt].hsi2c_sem));
//            }
//            retVal += I2c_GlobalDeInit(NULL);
//            if (BSP_SOK != retVal)
//            {
//                GT_0trace(BspAppTrace, GT_ERR, "Error: I2C De-Init failed!!\r\n");
//            }
//        }
    }

    retVal += Fvid2_deInit(NULL);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: FVID2 De-Init failed!!\r\n");
    }

    retVal += Bsp_platformDeInit(NULL);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR,
                  "Error: BSP Platform De-Init failed!!\r\n");
    }

    retVal += Bsp_boardDeInit(NULL);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: BSP Board De-Init failed!!\r\n");
    }

    retVal += Bsp_commonDeInit(NULL);
    if (BSP_SOK != retVal)
    {
        GT_0trace(BspAppTrace, GT_ERR, "Error: BSP Common De-Init failed!!\r\n");
    }

    return (retVal);
}
